@@ -1,40 +1,44 @@
 from __future__ import annotations
 
+import logging
 import os
 import re
+import shlex
+import subprocess
 import sys
 import typing as t
-import logging
-import subprocess
 from sys import version_info
-from shlex import quote
-from typing import TYPE_CHECKING
 
-import fs
 import attr
-import yaml
+import fs
+import jinja2
 import psutil
-import fs.copy
+import yaml
 from pathspec import PathSpec
 
-from ..utils import bentoml_cattr
-from ..utils import resolve_user_filepath
-from ..utils import copy_file_to_fs_folder
-from ..container import generate_containerfile
-from ...exceptions import InvalidArgument
 from ...exceptions import BentoMLException
-from ..utils.dotenv import parse_dotenv
-from ..configuration import CLEAN_BENTOML_VERSION
-from ..container.generate import BENTO_PATH
-from .build_dev_bentoml_whl import build_bentoml_editable_wheel
+from ...exceptions import InvalidArgument
+from ..configuration import DEFAULT_LOCK_PLATFORM
+from ..configuration import clean_bentoml_version
+from ..configuration import get_bentoml_requirement
+from ..configuration import get_debug_mode
+from ..configuration import get_quiet_mode
+from ..container import generate_containerfile
+from ..container.frontend.dockerfile import ALLOWED_CUDA_VERSION_ARGS
+from ..container.frontend.dockerfile import CONTAINER_SUPPORTED_DISTROS
+from ..container.frontend.dockerfile import SUPPORTED_CUDA_VERSIONS
 from ..container.frontend.dockerfile import DistroSpec
 from ..container.frontend.dockerfile import get_supported_spec
-from ..container.frontend.dockerfile import SUPPORTED_CUDA_VERSIONS
-from ..container.frontend.dockerfile import ALLOWED_CUDA_VERSION_ARGS
-from ..container.frontend.dockerfile import SUPPORTED_PYTHON_VERSIONS
-from ..container.frontend.dockerfile import CONTAINER_SUPPORTED_DISTROS
+from ..container.generate import BENTO_PATH
+from ..utils.args import set_arguments
+from ..utils.cattr import bentoml_cattr
+from ..utils.dotenv import parse_dotenv
+from ..utils.filesystem import copy_file_to_fs_folder
+from ..utils.filesystem import resolve_user_filepath
+from ..utils.uri import encode_path_for_uri
+from .bentoml_builder import build_git_repo
 
-if TYPE_CHECKING:
+if t.TYPE_CHECKING:
     from attr import Attribute
     from fs.base import FS
 
@@ -59,13 +63,13 @@ def _convert_python_version(py_version: str | None) -> str | None:
     if match is None:
         raise InvalidArgument(
             f'Invalid build option: docker.python_version="{py_version}", python '
-            f"version must follow standard python semver format, e.g. 3.7.10 ",
+            "version must follow standard python semver format, e.g. 3.8.15",
         )
     major, minor = match.groups()
     target_python_version = f"{major}.{minor}"
     if target_python_version != py_version:
         logger.warning(
-            "BentoML will install the latest 'python%s' instead of the specified 'python%s'. To use the exact python version, use a custom docker base image. See https://docs.bentoml.org/en/latest/concepts/bento.html#custom-base-image-advanced",
+            "BentoML will install the latest 'python%s' instead of the specified 'python%s'. To use the exact python version, use a custom docker base image. See https://docs.bentoml.com/en/latest/concepts/bento.html#custom-base-image-advanced",
             target_python_version,
             py_version,
         )
@@ -73,7 +77,7 @@ def _convert_python_version(py_version: str | None) -> str | None:
 
 
 def _convert_cuda_version(
-    cuda_version: t.Optional[t.Union[str, int]]
+    cuda_version: t.Optional[t.Union[str, int]],
 ) -> t.Optional[str]:
     if cuda_version is None or cuda_version == "" or cuda_version == "None":
         return None
@@ -95,9 +99,9 @@ def _convert_cuda_version(
 
 
 def _convert_env(
-    env: str | list[str] | dict[str, str] | None
+    env: str | list[str] | dict[str, str] | None,
 ) -> dict[str, str] | dict[str, str | None] | None:
-    if env is None:
+    if not env:
         return None
 
     if isinstance(env, str):
@@ -154,11 +158,7 @@ class DockerOptions:
         ),
     )
     python_version: t.Optional[str] = attr.field(
-        converter=_convert_python_version,
-        default=None,
-        validator=attr.validators.optional(
-            attr.validators.in_(SUPPORTED_PYTHON_VERSIONS)
-        ),
+        converter=_convert_python_version, default=None
     )
     cuda_version: t.Optional[str] = attr.field(
         default=None,
@@ -210,7 +210,9 @@ class DockerOptions:
                     f'Distro "{self.distro}" does not support CUDA. Distros that support CUDA are: {supports_cuda}.'
                 )
 
-    def with_defaults(self) -> DockerOptions:
+    def with_defaults(
+        self, default_envs: list[BentoEnvSchema] | None = None
+    ) -> DockerOptions:
         # Convert from user provided options to actual build options with default values
         defaults: t.Dict[str, t.Any] = {}
 
@@ -221,6 +223,9 @@ class DockerOptions:
                 python_version = f"{version_info.major}.{version_info.minor}"
                 defaults["python_version"] = python_version
 
+        if self.env is None and default_envs:
+            defaults["env"] = {e.name: e.value for e in default_envs}
+
         return attr.evolve(self, **defaults)
 
     def write_to_bento(self, bento_fs: FS, build_ctx: str, conda: CondaOptions) -> None:
@@ -278,7 +283,7 @@ class DockerOptions:
         return bentoml_cattr.unstructure(self)
 
 
-if TYPE_CHECKING:
+if t.TYPE_CHECKING:
     CondaPipType = dict[t.Literal["pip"], list[str]]
     DependencyType = list[str | CondaPipType]
 else:
@@ -312,7 +317,7 @@ def conda_dependencies_validator(
                 )
 
 
-if TYPE_CHECKING:
+if t.TYPE_CHECKING:
     ListStr: t.TypeAlias = list[str]
     CondaYamlDict = dict[str, DependencyType | list[str]]
 else:
@@ -459,10 +464,8 @@ class PythonOptions:
         default=None,
         validator=attr.validators.optional(attr.validators.instance_of(ListStr)),
     )
-    lock_packages: t.Optional[bool] = attr.field(
-        default=None,
-        validator=attr.validators.optional(attr.validators.instance_of(bool)),
-    )
+    lock_packages: t.Optional[bool] = None
+    pack_git_packages: t.Optional[bool] = None
     index_url: t.Optional[str] = attr.field(
         default=None,
         validator=attr.validators.optional(attr.validators.instance_of(str)),
@@ -495,7 +498,7 @@ class PythonOptions:
     def __attrs_post_init__(self):
         if self.requirements_txt and self.packages:
             logger.warning(
-                "Build option python: 'requirements_txt={self.requirements_txt}' found, will ignore the option: 'packages=%s'.",
+                "Build option python: 'requirements_txt=%s' found, will ignore the option: 'packages=%s'.",
                 self.requirements_txt,
                 self.packages,
             )
@@ -508,7 +511,25 @@ class PythonOptions:
     def is_empty(self) -> bool:
         return not self.requirements_txt and not self.packages
 
-    def write_to_bento(self, bento_fs: FS, build_ctx: str) -> None:
+    @property
+    def _jinja_environment(self) -> jinja2.Environment:
+        env = jinja2.Environment(
+            extensions=["jinja2.ext.debug"],
+            variable_start_string="<<",
+            variable_end_string=">>",
+            loader=jinja2.FileSystemLoader(os.path.dirname(__file__), followlinks=True),
+        )
+        env.filters["bash_quote"] = shlex.quote
+        return env
+
+    def write_to_bento(
+        self, bento_fs: FS, build_ctx: str, platform_: str | None = None
+    ) -> None:
+        import platform
+
+        from ..configuration import get_uv_command
+        from .bentoml_builder import build_bentoml_sdist
+
         py_folder = fs.path.join("env", "python")
         wheels_folder = fs.path.join(py_folder, "wheels")
         bento_fs.makedirs(py_folder, recreate=True)
@@ -517,8 +538,8 @@ class PythonOptions:
         with bento_fs.open(fs.path.join(py_folder, "version.txt"), "w") as f:
             f.write(f"{version_info.major}.{version_info.minor}.{version_info.micro}")
 
-        # Build BentoML whl from local source if BENTOML_BUNDLE_LOCAL_BUILD=True
-        build_bentoml_editable_wheel(bento_fs.getsyspath(wheels_folder))
+        # Build BentoML sdist from local source if BENTOML_BUNDLE_LOCAL_BUILD=True
+        sdist_name = build_bentoml_sdist(bento_fs.getsyspath(wheels_folder))
 
         # Move over required wheel files
         if self.wheels:
@@ -527,7 +548,7 @@ class PythonOptions:
                 whl_file = resolve_user_filepath(whl_file, build_ctx)
                 copy_file_to_fs_folder(whl_file, bento_fs, wheels_folder)
 
-        pip_compile_compat: t.List[str] = []
+        pip_compile_compat: list[str] = []
         if self.index_url:
             pip_compile_compat.extend(["--index-url", self.index_url])
         if self.trusted_host:
@@ -541,7 +562,7 @@ class PythonOptions:
                 pip_compile_compat.extend(["--extra-index-url", url])
 
         # add additional pip args that does not apply to pip-compile
-        pip_args: t.List[str] = []
+        pip_args: list[str] = []
         pip_args.extend(pip_compile_compat)
         if self.no_index:
             pip_args.append("--no-index")
@@ -549,87 +570,63 @@ class PythonOptions:
             pip_args.extend(self.pip_args.split())
 
         with bento_fs.open(fs.path.combine(py_folder, "install.sh"), "w") as f:
-            args = ["--no-warn-script-location"]
+            args: list[str] = []
             if pip_args:
                 args.extend(pip_args)
-            install_sh = (
-                """\
-#!/usr/bin/env bash
-set -exuo pipefail
-
-# Parent directory https://stackoverflow.com/a/246128/8643197
-BASEDIR="$( cd -- "$( dirname -- "${BASH_SOURCE[0]:-$0}"; )" &> /dev/null && pwd 2> /dev/null; )"
-
-PIP_ARGS=("""
-                + " ".join(map(quote, args))
-                + """)
-
-# BentoML by default generates two requirement files:
-#  - ./env/python/requirements.lock.txt: all dependencies locked to its version presented during `build`
-#  - ./env/python/requirements.txt: all dependencies as user specified in code or requirements.txt file
-REQUIREMENTS_TXT="$BASEDIR/requirements.txt"
-REQUIREMENTS_LOCK="$BASEDIR/requirements.lock.txt"
-WHEELS_DIR="$BASEDIR/wheels"
-BENTOML_VERSION=${BENTOML_VERSION:-"""
-                + CLEAN_BENTOML_VERSION
-                + """}
-# Install python packages, prefer installing the requirements.lock.txt file if it exist
-if [ -f "$REQUIREMENTS_LOCK" ]; then
-    echo "Installing pip packages from 'requirements.lock.txt'.."
-    pip3 install -r "$REQUIREMENTS_LOCK" "${PIP_ARGS[@]}"
-else
-    if [ -f "$REQUIREMENTS_TXT" ]; then
-        echo "Installing pip packages from 'requirements.txt'.."
-        pip3 install -r "$REQUIREMENTS_TXT" "${PIP_ARGS[@]}"
-    fi
-fi
-
-# Install user-provided wheels
-if [ -d "$WHEELS_DIR" ]; then
-    echo "Installing wheels packaged in Bento.."
-    pip3 install "$WHEELS_DIR"/*.whl "${PIP_ARGS[@]}"
-fi
-
-# Install the BentoML from PyPI if it's not already installed
-if python3 -c "import bentoml" &> /dev/null; then
-    existing_bentoml_version=$(python3 -c "import bentoml; print(bentoml.__version__)")
-    if [ "$existing_bentoml_version" != "$BENTOML_VERSION" ]; then
-        echo "WARNING: using BentoML version ${existing_bentoml_version}"
-    fi
-else
-    pip3 install bentoml=="$BENTOML_VERSION"
-fi
-"""
+            f.write(
+                self._jinja_environment.get_template("install.sh.j2").render(
+                    bentoml_version=clean_bentoml_version(),
+                    pip_args=shlex.join(args),
+                )
             )
-            f.write(install_sh)
 
-        if self.requirements_txt is not None:
-            from pip_requirements_parser import RequirementsFile
+        with bento_fs.open(fs.path.join(py_folder, "requirements.txt"), "w") as f:
+            has_bentoml_req = False
 
-            requirements_txt = RequirementsFile.from_file(
-                resolve_user_filepath(self.requirements_txt, build_ctx),
-                include_nested=True,
-            )
-            # We need to make sure that we don't write any file references
-            # back into the final `requirements.txt` file. We've already
-            # resolved them and included their contents so we can discard
-            # them.
-            for option_line in requirements_txt.options:
-                option_line.options.pop("constraints", None)
-                option_line.options.pop("requirements", None)
+            if self.requirements_txt is not None:
+                from pip_requirements_parser import RequirementsFile
+
+                requirements_txt = RequirementsFile.from_file(
+                    resolve_user_filepath(self.requirements_txt, build_ctx),
+                    include_nested=True,
+                )
+                # We need to make sure that we don't write any file references
+                # back into the final `requirements.txt` file. We've already
+                # resolved them and included their contents so we can discard
+                # them.
+                for option_line in requirements_txt.options:
+                    option_line.options.pop("constraints", None)
+                    option_line.options.pop("requirements", None)
+
+                if any(
+                    req.name and req.name.lower() == "bentoml" and req.link is not None
+                    for req in requirements_txt.requirements
+                ):
+                    has_bentoml_req = True
 
-            with bento_fs.open(
-                fs.path.combine(py_folder, "requirements.txt"), "w"
-            ) as f:
                 f.write(requirements_txt.dumps(preserve_one_empty_line=True))
-        elif self.packages is not None:
-            with bento_fs.open(fs.path.join(py_folder, "requirements.txt"), "w") as f:
-                f.write("\n".join(self.packages))
-        else:
-            # Return early if no python packages were specified
-            return
+            elif self.packages is not None:
+                bentoml_req_regex = re.compile(
+                    r"^bentoml(?:\[[^\]]+\])?\s*@", re.IGNORECASE
+                )
+                if any(bentoml_req_regex.match(pkg) for pkg in self.packages):
+                    has_bentoml_req = True
+                f.write("\n".join(self.packages) + "\n")
 
-        if self.lock_packages and not self.is_empty():
+            if not has_bentoml_req:
+                # Add the pinned BentoML requirement first if it's not a local version
+                if sdist_name is None and (bentoml_req := get_bentoml_requirement()):
+                    logger.info(
+                        "Adding current BentoML version to requirements.txt: %s",
+                        bentoml_req,
+                    )
+                    f.write(f"{bentoml_req}\n")
+                elif sdist_name:
+                    f.write(f"./wheels/{sdist_name}\n")
+
+            is_empty = f.tell() == 0
+
+        if self.lock_packages and not is_empty:
             # Note: "--allow-unsafe" is required for including setuptools in the
             # generated requirements.lock.txt file, and setuptool is required by
             # pyfilesystem2. Once pyfilesystem2 drop setuptools as dependency, we can
@@ -648,33 +645,100 @@ fi
             pip_compile_args.extend(pip_compile_compat)
             pip_compile_args.extend(
                 [
-                    "--quiet",
                     "--allow-unsafe",
                     "--no-header",
                     f"--output-file={pip_compile_out}",
-                    "--resolver=backtracking",
+                    "--emit-index-url",
+                    "--emit-find-links",
+                    "--no-annotate",
                 ]
             )
+            if get_debug_mode():
+                pip_compile_args.append("--verbose")
+            else:
+                pip_compile_args.append("--quiet")
             logger.info("Locking PyPI package versions.")
-            cmd = [sys.executable, "-m", "piptools", "compile"]
+            if platform_:
+                pip_compile_args.extend(["--python-platform", platform_])
+            elif platform.system() != "Linux" or platform.machine() != "x86_64":
+                logger.info(
+                    "Locking packages for %s. Pass `--platform` option to specify the platform.",
+                    DEFAULT_LOCK_PLATFORM,
+                )
+                pip_compile_args.extend(["--python-platform", DEFAULT_LOCK_PLATFORM])
+            cmd = [*get_uv_command(), "pip", "compile"]
             cmd.extend(pip_compile_args)
             try:
-                subprocess.check_call(cmd)
+                subprocess.check_call(
+                    cmd,
+                    text=True,
+                    stderr=subprocess.DEVNULL if get_quiet_mode() else None,
+                    cwd=bento_fs.getsyspath(py_folder),
+                )
             except subprocess.CalledProcessError as e:
-                logger.error("Failed to lock PyPI packages: %s", e, exc_info=e)
-                logger.error(
-                    "Falling back to using the user-provided package requirement specifiers, which is equivalent to 'lock_packages=false'."
+                raise BentoMLException(f"Failed to lock PyPI packages: {e}") from None
+            self.fix_dep_urls(
+                pip_compile_out,
+                bento_fs.getsyspath(wheels_folder),
+                self.pack_git_packages,
+            )
+        else:
+            requirements_txt = bento_fs.getsyspath(
+                fs.path.combine(py_folder, "requirements.txt")
+            )
+            if os.path.exists(requirements_txt):
+                self.fix_dep_urls(
+                    requirements_txt,
+                    bento_fs.getsyspath(wheels_folder),
+                    self.pack_git_packages,
                 )
 
     def with_defaults(self) -> PythonOptions:
         # Convert from user provided options to actual build options with default values
-        defaults: dict[str, t.Any] = {}
+        new_attrs = {}
+        if self.lock_packages is None:
+            new_attrs["lock_packages"] = self.pack_git_packages is not False
+        if self.pack_git_packages is None:
+            new_attrs["pack_git_packages"] = True
+        return attr.evolve(self, **new_attrs) if new_attrs else self
 
-        if self.requirements_txt is None:
-            if self.lock_packages is None:
-                defaults["lock_packages"] = True
+    @staticmethod
+    def fix_dep_urls(
+        requirements_txt: str, wheels_folder: str, pack_git_packages: bool = True
+    ) -> None:
+        """Replace the git dependencies in the requirements.lock file with the
+        paths to the local copy.
+        """
+        from pip_requirements_parser import RequirementsFile
+        from pip_requirements_parser import parse_reqparts_from_string
 
-        return attr.evolve(self, **defaults)
+        parsed_requirements = RequirementsFile.from_file(
+            requirements_txt, include_nested=True
+        )
+        for req in parsed_requirements.requirements:
+            link = req.link
+            if not link:
+                continue
+
+            if "/env/python/wheels" in link.url:
+                filename = link.filename
+            elif pack_git_packages and link.url.startswith("git+"):
+                # We are only able to handle SSH Git URLs
+                url, ref = link.url_without_fragment[4:], ""
+                if "@" in link.path:  # ssh://git@owner/repo@ref
+                    url, _, ref = url.rpartition("@")
+                filename = build_git_repo(
+                    url, ref, link.subdirectory_fragment, wheels_folder
+                )
+            else:
+                continue
+            parsed_parts = parse_reqparts_from_string(f"./wheels/{filename}")
+            req.link = parsed_parts.link
+            req.req = parsed_parts.requirement
+            req.requirement_line.line = f"./wheels/{filename}"
+
+        with open(requirements_txt, "w") as f:
+            f.write(parsed_requirements.dumps(preserve_one_empty_line=True))
 
 
 def _python_options_structure_hook(d: t.Any, _: t.Type[PythonOptions]) -> PythonOptions:
@@ -689,14 +753,14 @@ def _python_options_structure_hook(d: t.Any, _: t.Type[PythonOptions]) -> Python
 bentoml_cattr.register_structure_hook(PythonOptions, _python_options_structure_hook)
 
 
-if TYPE_CHECKING:
-    OptionsCls = DockerOptions | CondaOptions | PythonOptions
+if t.TYPE_CHECKING:
+    OptionsCls = t.TypeVar("OptionsCls", DockerOptions, CondaOptions, PythonOptions)
 
 
 def dict_options_converter(
     options_type: t.Type[OptionsCls],
-) -> t.Callable[[OptionsCls | dict[str, t.Any]], t.Any]:
-    def _converter(value: OptionsCls | dict[str, t.Any]) -> options_type:
+) -> t.Callable[[OptionsCls | dict[str, t.Any] | None], OptionsCls]:
+    def _converter(value: OptionsCls | dict[str, t.Any] | None) -> OptionsCls:
         if value is None:
             return options_type()
         if isinstance(value, dict):
@@ -706,6 +770,45 @@ def dict_options_converter(
     return _converter
 
 
+@attr.frozen
+class ModelSpec:
+    tag: str
+    filter: t.Optional[str] = None
+    alias: t.Optional[str] = None
+
+    @classmethod
+    def from_item(cls, item: str | dict[str, t.Any] | ModelSpec) -> ModelSpec:
+        if isinstance(item, str):
+            return cls(tag=item)
+        if isinstance(item, ModelSpec):
+            return item
+        return cls(**item)
+
+
+def convert_models_config(
+    models_config: list[str | dict[str, t.Any] | ModelSpec] | None,
+) -> list[ModelSpec]:
+    if not models_config:
+        return []
+    return [ModelSpec.from_item(item) for item in models_config]
+
+
+def _model_spec_structure_hook(
+    d: str | dict[str, t.Any], cls: t.Type[ModelSpec]
+) -> ModelSpec:
+    return cls.from_item(d)
+
+
+@attr.define(eq=True)
+class BentoEnvSchema:
+    __forbid_extra_keys__ = False
+    name: str
+    value: str = ""
+
+
+bentoml_cattr.register_structure_hook(ModelSpec, _model_spec_structure_hook)
+
+
 @attr.frozen
 class BentoBuildConfig:
     """This class is intended for modeling the bentofile.yaml file where user will
@@ -721,10 +824,10 @@ class BentoBuildConfig:
     # no need to omit since BentoML has already handled the default values.
     __omit_if_default__ = False
 
-    service: str
+    service: str = ""
     name: t.Optional[str] = None
     description: t.Optional[str] = None
-    labels: t.Optional[t.Dict[str, t.Any]] = None
+    labels: t.Dict[str, str] = attr.field(factory=dict)
     include: t.Optional[t.List[str]] = None
     exclude: t.Optional[t.List[str]] = None
     docker: DockerOptions = attr.field(
@@ -739,26 +842,11 @@ class BentoBuildConfig:
         default=None,
         converter=dict_options_converter(CondaOptions),
     )
-
-    if TYPE_CHECKING:
-        # NOTE: This is to ensure that BentoBuildConfig __init__
-        # satisfies type checker. docker, python, and conda accepts
-        # dict[str, t.Any] since our converter will handle the conversion.
-        # There is no way to tell type checker signatures of the converter from attrs
-        # if given attribute is alrady has a type annotation.
-        def __init__(
-            self,
-            service: str,
-            name: str | None = ...,
-            description: str | None = ...,
-            labels: dict[str, t.Any] | None = ...,
-            include: list[str] | None = ...,
-            exclude: list[str] | None = ...,
-            docker: DockerOptions | dict[str, t.Any] | None = ...,
-            python: PythonOptions | dict[str, t.Any] | None = ...,
-            conda: CondaOptions | dict[str, t.Any] | None = ...,
-        ) -> None:
-            ...
+    models: t.List[ModelSpec] = attr.field(
+        factory=list, converter=convert_models_config
+    )
+    envs: t.List[BentoEnvSchema] = attr.field(factory=list)
+    args: t.Dict[str, t.Any] = attr.field(factory=dict)
 
     def __attrs_post_init__(self) -> None:
         use_conda = not self.conda.is_empty()
@@ -766,7 +854,7 @@ class BentoBuildConfig:
 
         if use_cuda and use_conda:
             logger.warning(
-                "BentoML does not support using both conda dependencies and setting a CUDA version for GPU. If you need both conda and CUDA, use a custom base image or create a dockerfile_template, see https://docs.bentoml.org/en/latest/concepts/bento.html#custom-base-image-advanced"
+                "BentoML does not support using both conda dependencies and setting a CUDA version for GPU. If you need both conda and CUDA, use a custom base image or create a dockerfile_template, see https://docs.bentoml.com/en/latest/concepts/bento.html#custom-base-image-advanced"
             )
 
         if self.docker.distro is not None:
@@ -796,6 +884,11 @@ class BentoBuildConfig:
                         f"{self.docker.cuda_version} is not supported for {self.docker.distro}. Supported cuda versions are: {', '.join(spec.supported_cuda_versions)}."
                     )
 
+        if self.args:
+            # Merge command line args with the ones in the config file
+            # and the command line args take precedence
+            set_arguments(self.args, override=False)
+
     def with_defaults(self) -> FilledBentoBuildConfig:
         """
         Convert from user provided options to actual build options with defaults
@@ -805,17 +898,24 @@ class BentoBuildConfig:
             BentoBuildConfig: a new copy of self, with default values filled
         """
         return FilledBentoBuildConfig(
-            self.service,
-            self.name,
-            self.description,
-            {} if self.labels is None else self.labels,
-            ["*"] if self.include is None else self.include,
-            [] if self.exclude is None else self.exclude,
-            self.docker.with_defaults(),
-            self.python.with_defaults(),
-            self.conda.with_defaults(),
+            service=self.service,
+            name=self.name,
+            description=self.description,
+            labels=self.labels,
+            include=["*"] if self.include is None else self.include,
+            exclude=[] if self.exclude is None else self.exclude,
+            docker=self.docker.with_defaults(self.envs),
+            python=self.python.with_defaults(),
+            conda=self.conda.with_defaults(),
+            models=self.models,
+            envs=self.envs,
+            args=self.args,
         )
 
+    @property
+    def model_aliases(self) -> t.Dict[str, str]:
+        return {model.alias: model.tag for model in self.models if model.alias}
+
     @classmethod
     def from_yaml(cls, stream: t.TextIO) -> BentoBuildConfig:
         try:
@@ -823,74 +923,114 @@ class BentoBuildConfig:
         except yaml.YAMLError as exc:
             logger.error(exc)
             raise
+        return cls.load(yaml_content)
 
-        try:
-            return bentoml_cattr.structure(yaml_content, cls)
-        except TypeError as e:
-            if "missing 1 required positional argument: 'service'" in str(e):
-                raise InvalidArgument(
-                    'Missing required build config field "service", which indicates import path of target bentoml.Service instance. e.g.: "service: fraud_detector.py:svc"'
-                ) from e
-            else:
-                raise InvalidArgument(str(e)) from e
+    @classmethod
+    def from_pyproject(cls, stream: t.BinaryIO) -> BentoBuildConfig:
+        if sys.version_info >= (3, 11):
+            import tomllib
+        else:
+            import tomli as tomllib
+        data = tomllib.load(stream)
+        config = data.get("tool", {}).get("bentoml", {}).get("build", {})
+        if "name" in data.get("project", {}):
+            config.setdefault("name", data["project"]["name"])
+        build_config = cls.load(config)
+        dependencies = data.get("project", {}).get("dependencies", {})
+        python_packages = build_config.python.packages or []
+        python_packages.extend(dependencies)
+        object.__setattr__(build_config.python, "packages", python_packages)
+        return build_config
+
+    @classmethod
+    def from_file(cls, path: str) -> BentoBuildConfig:
+        if os.path.basename(path) == "pyproject.toml":
+            with open(path, "rb") as f:
+                return cls.from_pyproject(f)
+        else:
+            with open(path, encoding="utf-8") as f:
+                return cls.from_yaml(f)
+
+    @classmethod
+    def load(cls, data: dict[str, t.Any]) -> BentoBuildConfig:
+        return bentoml_cattr.structure(data, cls)
+
+    @classmethod
+    def from_bento_dir(cls, bento_dir: str) -> BentoBuildConfig:
+        from .bento import DEFAULT_BENTO_BUILD_FILES
+
+        for filename in DEFAULT_BENTO_BUILD_FILES:
+            bentofile_path = os.path.join(bento_dir, filename)
+            if os.path.exists(bentofile_path):
+                return cls.from_file(bentofile_path).with_defaults()
+        return cls(service="").with_defaults()
 
     def to_yaml(self, stream: t.TextIO) -> None:
-        # TODO: Save BentoBuildOptions to a yaml file
-        # This is reserved for building interactive build file creation CLI
-        raise NotImplementedError
+        try:
+            yaml.dump(bentoml_cattr.unstructure(self), stream)
+        except yaml.YAMLError as e:
+            logger.error("Error while deserializing BentoBuildConfig to yaml:")
+            logger.error(e)
+            raise
 
 
-@attr.define(frozen=True)
+@attr.frozen
 class BentoPathSpec:
-    _include: PathSpec = attr.field(
+    include: PathSpec = attr.field(
         converter=lambda x: PathSpec.from_lines("gitwildmatch", x)
     )
-    _exclude: PathSpec = attr.field(
+    exclude: PathSpec = attr.field(
         converter=lambda x: PathSpec.from_lines("gitwildmatch", x)
     )
-    # we want to ignore .git folder in cases the .git folder is very large.
-    git: PathSpec = attr.field(
-        default=PathSpec.from_lines("gitwildmatch", [".git"]), init=False
+    ctx_dir: str = attr.field(default=".")
+    recurse_ignore_filename: str = ".bentoignore"
+    recurse_exclude_spec: list[tuple[str, PathSpec]] = attr.field(init=False)
+    # we want to ignore .git and venv folders in cases they are very large.
+    extra: PathSpec = attr.field(
+        default=PathSpec.from_lines(
+            "gitwildmatch", [".git/", ".venv/", "venv/", "__pycache__/", ".DS_Store"]
+        ),
+        init=False,
     )
 
-    def includes(
-        self,
-        path: str,
-        *,
-        recurse_exclude_spec: t.Optional[t.Iterable[t.Tuple[str, PathSpec]]] = None,
-    ) -> bool:
-        # Determine whether a path is included or not.
+    @recurse_exclude_spec.default
+    def _default_recurse_exclude_spec(self) -> list[tuple[str, PathSpec]]:
         # recurse_exclude_spec is a list of (path, spec) pairs.
+        fs_ = fs.open_fs(encode_path_for_uri(self.ctx_dir))
+        recurse_exclude_spec: list[tuple[str, PathSpec]] = []
+        for file in fs_.walk.files(filter=[self.recurse_ignore_filename]):
+            dir_path = fs.path.dirname(file)
+            with fs_.open(file) as f:
+                recurse_exclude_spec.append(
+                    (dir_path.lstrip("/"), PathSpec.from_lines("gitwildmatch", f))
+                )
+        return recurse_exclude_spec
+
+    def includes(self, path: str) -> bool:
+        """Determine whether a path is included or not."""
         to_include = (
-            self._include.match_file(path)
-            and not self._exclude.match_file(path)
-            and not self.git.match_file(path)
+            self.include.match_file(path)
+            and not self.exclude.match_file(path)
+            and not self.extra.match_file(path)
         )
         if to_include:
-            if recurse_exclude_spec is not None:
-                return not any(
-                    ignore_spec.match_file(fs.path.relativefrom(ignore_parent, path))
-                    for ignore_parent, ignore_spec in recurse_exclude_spec
-                )
-        return False
-
-    def from_path(self, path: str) -> t.Generator[t.Tuple[str, PathSpec], None, None]:
-        """
-        yield (parent, exclude_spec) from .bentoignore file of a given path
-        """
-        fs_ = fs.open_fs(path)
-        for file in fs_.walk.files(filter=[".bentoignore"]):
-            dir_path = "".join(fs.path.parts(file)[:-1])
-            yield dir_path, PathSpec.from_lines("gitwildmatch", fs_.open(file))
+            return not any(
+                ignore_spec.match_file(fs.path.relativefrom(ignore_parent, path))
+                for ignore_parent, ignore_spec in self.recurse_exclude_spec
+                if fs.path.isparent(ignore_parent, path)
+            )
+        return to_include
 
 
 class FilledBentoBuildConfig(BentoBuildConfig):
     service: str
     name: t.Optional[str]
     description: t.Optional[str]
-    labels: t.Dict[str, t.Any]
+    labels: t.Dict[str, str]
     include: t.List[str]
     exclude: t.List[str]
     docker: DockerOptions
     python: PythonOptions
     conda: CondaOptions
+    models: t.List[ModelSpec]
+    envs: t.List[BentoEnvSchema]
