@@ -1,6 +1,7 @@
 """
 This module is shim for bentoctl. NOT FOR DIRECT USE.
 """
+
 from __future__ import annotations
 
 import logging
@@ -10,6 +11,7 @@ from typing import TYPE_CHECKING
 import fs
 
 from ..container.generate import generate_containerfile
+from ..utils.uri import encode_path_for_uri
 
 if TYPE_CHECKING:
     from .build_config import DockerOptions
@@ -29,7 +31,7 @@ warnings.warn(
 def generate_dockerfile(docker: DockerOptions, context_path: str, *, use_conda: bool):
     from ..bento import Bento
 
-    bento = Bento.from_fs(fs.open_fs(context_path))
+    bento = Bento.from_fs(fs.open_fs(encode_path_for_uri(context_path)))
     logger.debug("'use_conda' is deprecated and will not be used.")
     return generate_containerfile(
         docker,
