@@ -1,22 +1,22 @@
 from __future__ import annotations
 
-import typing as t
 import asyncio
+import typing as t
 
+from python_multipart.multipart import parse_options_header
 from starlette.requests import Request
-from multipart.multipart import parse_options_header
 from starlette.responses import Response
 
-from . import from_spec as io_descriptor_from_spec
-from .base import IODescriptor
-from ...exceptions import InvalidArgument
 from ...exceptions import BentoMLException
+from ...exceptions import InvalidArgument
 from ...grpc.utils import import_generated_stubs
 from ..service.openapi import SUCCESS_DESCRIPTION
-from ..utils.formparser import populate_multipart_requests
-from ..utils.formparser import concat_to_multipart_response
-from ..service.openapi.specification import Schema
 from ..service.openapi.specification import MediaType
+from ..service.openapi.specification import Schema
+from ..utils.formparser import concat_to_multipart_response
+from ..utils.formparser import populate_multipart_requests
+from . import from_spec as io_descriptor_from_spec
+from .base import IODescriptor
 
 if t.TYPE_CHECKING:
     from types import UnionType
@@ -26,9 +26,9 @@ if t.TYPE_CHECKING:
     from bentoml.grpc.v1 import service_pb2 as pb
     from bentoml.grpc.v1alpha1 import service_pb2 as pb_v1alpha1
 
-    from .base import OpenAPIResponse
+    from ..context import ServiceContext as Context
     from ..types import LazyType
-    from ..context import InferenceApiContext as Context
+    from .base import OpenAPIResponse
 else:
     pb, _ = import_generated_stubs("v1")
     pb_v1alpha1, _ = import_generated_stubs("v1alpha1")
@@ -63,7 +63,7 @@ class Multipart(
 
        runner = bentoml.sklearn.get("sklearn_model_clf").to_runner()
 
-       svc = bentoml.Service("iris-classifier", runners=[runner])
+       svc = bentoml.legacy.Service("iris-classifier", runners=[runner])
 
        input_spec = Multipart(arr=NumpyNdarray(), annotations=JSON())
        output_spec = Multipart(output=NumpyNdarray(), result=JSON())
@@ -176,7 +176,7 @@ class Multipart(
         self._inputs = inputs
 
     def __repr__(self) -> str:
-        return f"Multipart({','.join([f'{k}={v}' for k,v in zip(self._inputs, map(repr, self._inputs.values()))])})"
+        return f"Multipart({','.join([f'{k}={v}' for k, v in zip(self._inputs, map(repr, self._inputs.values()))])})"
 
     def _from_sample(cls, sample: dict[str, t.Any]) -> t.Any:
         raise NotImplementedError("'from_sample' is not supported for Multipart.")
@@ -327,14 +327,12 @@ class Multipart(
     @t.overload
     async def _to_proto_impl(
         self, obj: dict[str, t.Any], *, version: t.Literal["v1"]
-    ) -> pb.Multipart:
-        ...
+    ) -> pb.Multipart: ...
 
     @t.overload
     async def _to_proto_impl(
         self, obj: dict[str, t.Any], *, version: t.Literal["v1alpha1"]
-    ) -> pb_v1alpha1.Multipart:
-        ...
+    ) -> pb_v1alpha1.Multipart: ...
 
     async def _to_proto_impl(
         self, obj: dict[str, t.Any], *, version: str
