@@ -4,26 +4,40 @@ User facing python APIs for managing local bentos and build new bentos.
 
 from __future__ import annotations
 
-import sys
-import typing as t
 import logging
-import subprocess
+import typing as t
 
-from simple_di import inject
+import attr
 from simple_di import Provide
+from simple_di import inject
+
+from bentoml._internal.bento.bento import DEFAULT_BENTO_BUILD_FILES
 
-from .exceptions import InvalidArgument
-from .exceptions import BentoMLException
-from ._internal.tag import Tag
 from ._internal.bento import Bento
-from ._internal.utils import resolve_user_filepath
 from ._internal.bento.build_config import BentoBuildConfig
 from ._internal.configuration.containers import BentoMLContainer
+from ._internal.tag import Tag
+from ._internal.utils.args import set_arguments
+from ._internal.utils.filesystem import resolve_user_filepath
+from .exceptions import BadInput
+from .exceptions import BentoMLException
+from .exceptions import InvalidArgument
 
 if t.TYPE_CHECKING:
+    from _bentoml_sdk import Service as NewService
+
     from ._internal.bento import BentoStore
-    from ._internal.yatai_client import YataiClient
-    from ._internal.server.server import ServerHandle
+    from ._internal.bento.build_config import BentoEnvSchema
+    from ._internal.bento.build_config import CondaOptions
+    from ._internal.bento.build_config import DockerOptions
+    from ._internal.bento.build_config import ModelSpec
+    from ._internal.bento.build_config import PythonOptions
+    from ._internal.cloud import BentoCloudClient
+    from ._internal.service import Service
+    from ._internal.utils.circus import Server
+
+    Servable = str | Bento | Tag | Service | NewService[t.Any]
+
 
 logger = logging.getLogger(__name__)
 
@@ -42,27 +56,27 @@ __all__ = [
 
 
 @inject
-def list(  # pylint: disable=redefined-builtin
-    tag: t.Optional[t.Union[Tag, str]] = None,
-    _bento_store: "BentoStore" = Provide[BentoMLContainer.bento_store],
-) -> "t.List[Bento]":
+def list(
+    tag: Tag | str | None = None,
+    _bento_store: BentoStore = Provide[BentoMLContainer.bento_store],
+) -> t.List[Bento]:
     return _bento_store.list(tag)
 
 
 @inject
 def get(
-    tag: t.Union[Tag, str],
+    tag: Tag | str,
     *,
-    _bento_store: "BentoStore" = Provide[BentoMLContainer.bento_store],
+    _bento_store: BentoStore = Provide[BentoMLContainer.bento_store],
 ) -> Bento:
     return _bento_store.get(tag)
 
 
 @inject
 def delete(
-    tag: t.Union[Tag, str],
+    tag: Tag | str,
     *,
-    _bento_store: "BentoStore" = Provide[BentoMLContainer.bento_store],
+    _bento_store: BentoStore = Provide[BentoMLContainer.bento_store],
 ):
     _bento_store.delete(tag)
 
@@ -70,13 +84,13 @@ def delete(
 @inject
 def import_bento(
     path: str,
-    input_format: t.Optional[str] = None,
+    input_format: str | None = None,
     *,
-    protocol: t.Optional[str] = None,
-    user: t.Optional[str] = None,
-    passwd: t.Optional[str] = None,
+    protocol: str | None = None,
+    user: str | None = None,
+    passwd: str | None = None,
     params: t.Optional[t.Dict[str, str]] = None,
-    subpath: t.Optional[str] = None,
+    subpath: str | None = None,
     _bento_store: "BentoStore" = Provide[BentoMLContainer.bento_store],
 ) -> Bento:
     """
@@ -150,16 +164,16 @@ def import_bento(
 
 @inject
 def export_bento(
-    tag: t.Union[Tag, str],
+    tag: Tag | str,
     path: str,
-    output_format: t.Optional[str] = None,
+    output_format: str | None = None,
     *,
-    protocol: t.Optional[str] = None,
-    user: t.Optional[str] = None,
-    passwd: t.Optional[str] = None,
-    params: t.Optional[t.Dict[str, str]] = None,
-    subpath: t.Optional[str] = None,
-    _bento_store: "BentoStore" = Provide[BentoMLContainer.bento_store],
+    protocol: str | None = None,
+    user: str | None = None,
+    passwd: str | None = None,
+    params: dict[str, str] | None = None,
+    subpath: str | None = None,
+    _bento_store: BentoStore = Provide[BentoMLContainer.bento_store],
 ) -> str:
     """
     Export a bento.
@@ -234,28 +248,28 @@ def export_bento(
 
 @inject
 def push(
-    tag: t.Union[Tag, str],
+    tag: Tag | str,
     *,
     force: bool = False,
-    _bento_store: "BentoStore" = Provide[BentoMLContainer.bento_store],
-    _yatai_client: YataiClient = Provide[BentoMLContainer.yatai_client],
+    _bento_store: BentoStore = Provide[BentoMLContainer.bento_store],
+    _cloud_client: BentoCloudClient = Provide[BentoMLContainer.bentocloud_client],
 ):
     """Push Bento to a yatai server."""
     bento = _bento_store.get(tag)
     if not bento:
         raise BentoMLException(f"Bento {tag} not found in local store")
-    _yatai_client.push_bento(bento, force=force)
+    _cloud_client.bento.push(bento, force=force)
 
 
 @inject
 def pull(
-    tag: t.Union[Tag, str],
+    tag: Tag | str,
     *,
     force: bool = False,
-    _bento_store: "BentoStore" = Provide[BentoMLContainer.bento_store],
-    _yatai_client: YataiClient = Provide[BentoMLContainer.yatai_client],
+    _bento_store: BentoStore = Provide[BentoMLContainer.bento_store],
+    _cloud_client: BentoCloudClient = Provide[BentoMLContainer.bentocloud_client],
 ):
-    _yatai_client.pull_bento(tag, force=force, bento_store=_bento_store)
+    _cloud_client.bento.pull(tag, force=force, bento_store=_bento_store)
 
 
 @inject
@@ -265,15 +279,19 @@ def build(
     name: str | None = None,
     labels: dict[str, str] | None = None,
     description: str | None = None,
-    include: list[str] | None = None,
-    exclude: list[str] | None = None,
-    docker: dict[str, t.Any] | None = None,
-    python: dict[str, t.Any] | None = None,
-    conda: dict[str, t.Any] | None = None,
+    include: t.List[str] | None = None,
+    exclude: t.List[str] | None = None,
+    envs: t.List[BentoEnvSchema] | None = None,
+    docker: DockerOptions | dict[str, t.Any] | None = None,
+    python: PythonOptions | dict[str, t.Any] | None = None,
+    conda: CondaOptions | dict[str, t.Any] | None = None,
+    models: t.List[ModelSpec | str | dict[str, t.Any]] | None = None,
     version: str | None = None,
     build_ctx: str | None = None,
+    platform: str | None = None,
+    args: dict[str, t.Any] | None = None,
     _bento_store: BentoStore = Provide[BentoMLContainer.bento_store],
-) -> "Bento":
+) -> Bento:
     """
     User-facing API for building a Bento. The available build options are identical to the keys of a
     valid 'bentofile.yaml' file.
@@ -297,8 +315,7 @@ def build(
                :class:`bentoml._internal.bento.build_config.CondaOptions`
         version: Override the default auto generated version str
         build_ctx: Build context directory, when used as
-        _bento_store: save Bento created to this BentoStore
-
+        platform: Platform to build for
     Returns:
         Bento: a Bento instance representing the materialized Bento saved in BentoStore
 
@@ -337,33 +354,45 @@ def build(
            )
 
     """
+    if args is not None:
+        set_arguments(args)
     build_config = BentoBuildConfig(
         service=service,
         name=name,
         description=description,
-        labels=labels,
+        labels=labels or {},
         include=include,
         exclude=exclude,
+        envs=envs or [],
         docker=docker,
         python=python,
         conda=conda,
+        models=models or [],
     )
 
     return Bento.create(
         build_config=build_config,
         version=version,
         build_ctx=build_ctx,
+        platform=platform,
     ).save(_bento_store)
 
 
 @inject
 def build_bentofile(
-    bentofile: str = "bentofile.yaml",
+    bentofile: str | None = None,
     *,
-    version: t.Optional[str] = None,
-    build_ctx: t.Optional[str] = None,
-    _bento_store: "BentoStore" = Provide[BentoMLContainer.bento_store],
-) -> "Bento":
+    service: str | None = None,
+    name: str | None = None,
+    version: str | None = None,
+    labels: dict[str, str] | None = None,
+    build_ctx: str | None = None,
+    platform: str | None = None,
+    bare: bool = False,
+    reload: bool = False,
+    args: dict[str, t.Any] | None = None,
+    _bento_store: BentoStore = Provide[BentoMLContainer.bento_store],
+) -> Bento:
     """
     Build a Bento base on options specified in a bentofile.yaml file.
 
@@ -374,21 +403,53 @@ def build_bentofile(
         bentofile: The file path to build config yaml file
         version: Override the default auto generated version str
         build_ctx: Build context directory, when used as
-        _bento_store: save Bento created to this BentoStore
+        bare: whether to build a bento without copying files
+        reload: whether to reload the service
+
+    Returns:
+        Bento: a Bento instance representing the materialized Bento saved in BentoStore
     """
-    try:
-        bentofile = resolve_user_filepath(bentofile, build_ctx)
-    except FileNotFoundError:
-        raise InvalidArgument(f'bentofile "{bentofile}" not found')
+    if args is not None:
+        set_arguments(args)
+    if bentofile:
+        try:
+            bentofile = resolve_user_filepath(bentofile, None)
+        except FileNotFoundError:
+            raise InvalidArgument(f'bentofile "{bentofile}" not found')
+        else:
+            build_config = BentoBuildConfig.from_file(bentofile)
+    else:
+        for filename in DEFAULT_BENTO_BUILD_FILES:
+            try:
+                bentofile = resolve_user_filepath(filename, build_ctx)
+            except FileNotFoundError:
+                pass
+            else:
+                build_config = BentoBuildConfig.from_file(bentofile)
+                break
+        else:
+            build_config = BentoBuildConfig(service=service or "")
 
-    with open(bentofile, "r", encoding="utf-8") as f:
-        build_config = BentoBuildConfig.from_yaml(f)
+    new_attrs = {}
+    if name is not None:
+        new_attrs["name"] = name
+    if labels:
+        new_attrs["labels"] = {**(build_config.labels or {}), **labels}
 
-    return Bento.create(
+    if new_attrs:
+        build_config = attr.evolve(build_config, **new_attrs)
+
+    bento = Bento.create(
         build_config=build_config,
         version=version,
         build_ctx=build_ctx,
-    ).save(_bento_store)
+        platform=platform,
+        bare=bare,
+        reload=reload,
+    )
+    if not bare:
+        return bento.save(_bento_store)
+    return bento
 
 
 def containerize(bento_tag: Tag | str, **kwargs: t.Any) -> bool:
@@ -417,105 +478,126 @@ def containerize(bento_tag: Tag | str, **kwargs: t.Any) -> bool:
         return False
 
 
-@inject
 def serve(
-    bento: str | Tag | Bento,
+    bento: Servable,
     server_type: str = "http",
     reload: bool = False,
-    production: bool = False,
+    production: bool = True,
     env: t.Literal["conda"] | None = None,
     host: str | None = None,
     port: int | None = None,
-    working_dir: str | None = None,
-    api_workers: int | None = Provide[BentoMLContainer.api_server_workers],
-    backlog: int = Provide[BentoMLContainer.api_server_config.backlog],
-    ssl_certfile: str | None = Provide[BentoMLContainer.ssl.certfile],
-    ssl_keyfile: str | None = Provide[BentoMLContainer.ssl.keyfile],
-    ssl_keyfile_password: str | None = Provide[BentoMLContainer.ssl.keyfile_password],
-    ssl_version: int | None = Provide[BentoMLContainer.ssl.version],
-    ssl_cert_reqs: int | None = Provide[BentoMLContainer.ssl.cert_reqs],
-    ssl_ca_certs: str | None = Provide[BentoMLContainer.ssl.ca_certs],
-    ssl_ciphers: str | None = Provide[BentoMLContainer.ssl.ciphers],
-    enable_reflection: bool = Provide[BentoMLContainer.grpc.reflection.enabled],
-    enable_channelz: bool = Provide[BentoMLContainer.grpc.channelz.enabled],
-    max_concurrent_streams: int
-    | None = Provide[BentoMLContainer.grpc.max_concurrent_streams],
+    working_dir: str = ".",
+    api_workers: int | None = None,
+    backlog: int | None = None,
+    ssl_certfile: str | None = None,
+    ssl_keyfile: str | None = None,
+    ssl_keyfile_password: str | None = None,
+    ssl_version: int | None = None,
+    ssl_cert_reqs: int | None = None,
+    ssl_ca_certs: str | None = None,
+    ssl_ciphers: str | None = None,
+    enable_reflection: bool | None = None,
+    enable_channelz: bool | None = None,
+    max_concurrent_streams: int | None = None,
     grpc_protocol_version: str | None = None,
-) -> ServerHandle:
-    from .serve import construct_ssl_args
-    from ._internal.server.server import ServerHandle
+    blocking: bool = False,
+    args: dict[str, t.Any] | None = None,
+) -> Server:
+    from ._internal.log import configure_logging
+    from ._internal.service import Service
+
+    if args is not None:
+        set_arguments(args, override=False)
 
     if isinstance(bento, Bento):
         bento = str(bento.tag)
     elif isinstance(bento, Tag):
         bento = str(bento)
 
-    server_type = server_type.lower()
-    if server_type not in ["http", "grpc"]:
-        raise ValueError('Server type must either be "http" or "grpc"')
-
-    ssl_args: dict[str, t.Any] = {
-        "ssl_certfile": ssl_certfile,
-        "ssl_keyfile": ssl_keyfile,
-        "ssl_ca_certs": ssl_ca_certs,
-    }
+    configure_logging()
     if server_type == "http":
-        serve_cmd = "serve-http"
-        if host is None:
-            host = BentoMLContainer.http.host.get()
-        if port is None:
-            port = BentoMLContainer.http.port.get()
+        from _bentoml_sdk import Service as NewService
 
-        ssl_args.update(
-            ssl_keyfile_password=ssl_keyfile_password,
-            ssl_version=ssl_version,
-            ssl_cert_reqs=ssl_cert_reqs,
-            ssl_ciphers=ssl_ciphers,
+        from ._internal.service import load
+
+        if not isinstance(bento, (Service, NewService)):
+            svc = load(bento, working_dir=working_dir)
+        else:
+            svc = bento
+
+        if isinstance(svc, Service):  # < 1.2 bento
+            from .serving import serve_http_production
+
+            if not isinstance(bento, str):
+                bento, working_dir = svc.get_service_import_origin()
+
+            return serve_http_production(
+                bento_identifier=bento,
+                reload=reload,
+                host=host,
+                port=port,
+                development_mode=not production,
+                working_dir=working_dir,
+                api_workers=api_workers,
+                backlog=backlog,
+                ssl_certfile=ssl_certfile,
+                ssl_keyfile=ssl_keyfile,
+                ssl_keyfile_password=ssl_keyfile_password,
+                ssl_version=ssl_version,
+                ssl_cert_reqs=ssl_cert_reqs,
+                ssl_ca_certs=ssl_ca_certs,
+                ssl_ciphers=ssl_ciphers,
+                threaded=not blocking,
+            )
+        else:  # >= 1.2 bento
+            from _bentoml_impl.server.serving import serve_http
+
+            if not isinstance(bento, str):
+                bento = svc.import_string
+                working_dir = svc.working_dir
+
+            svc.inject_config()
+            return serve_http(
+                bento_identifier=bento,
+                working_dir=working_dir,
+                reload=reload,
+                host=host,
+                port=port,
+                backlog=backlog,
+                development_mode=not production,
+                threaded=not blocking,
+                ssl_certfile=ssl_certfile,
+                ssl_keyfile=ssl_keyfile,
+                ssl_keyfile_password=ssl_keyfile_password,
+                ssl_version=ssl_version,
+                ssl_cert_reqs=ssl_cert_reqs,
+                ssl_ca_certs=ssl_ca_certs,
+                ssl_ciphers=ssl_ciphers,
+            )
+    elif server_type == "grpc":
+        from .serving import serve_grpc_production
+
+        if not isinstance(bento, str):
+            assert isinstance(bento, Service)
+            bento, working_dir = bento.get_service_import_origin()
+
+        return serve_grpc_production(
+            bento_identifier=bento,
+            reload=reload,
+            host=host,
+            port=port,
+            working_dir=working_dir,
+            api_workers=api_workers,
+            backlog=backlog,
+            threaded=not blocking,
+            development_mode=not production,
+            ssl_certfile=ssl_certfile,
+            ssl_keyfile=ssl_keyfile,
+            ssl_ca_certs=ssl_ca_certs,
+            max_concurrent_streams=max_concurrent_streams,
+            reflection=enable_reflection,
+            channelz=enable_channelz,
+            protocol_version=grpc_protocol_version,
         )
     else:
-        serve_cmd = "serve-grpc"
-        if host is None:
-            host = BentoMLContainer.grpc.host.get()
-        if port is None:
-            port = BentoMLContainer.grpc.port.get()
-
-    assert host is not None and port is not None
-    args: t.List[str] = [
-        sys.executable,
-        "-m",
-        "bentoml",
-        serve_cmd,
-        bento,
-        "--host",
-        host,
-        "--port",
-        str(port),
-        "--backlog",
-        str(backlog),
-        *construct_ssl_args(**ssl_args),
-    ]
-    if production:
-        args.append("--production")
-    if reload:
-        args.append("--reload")
-    if env:
-        args.extend(["--env", env])
-
-    if api_workers is not None:
-        args.extend(["--api-workers", str(api_workers)])
-    if working_dir is not None:
-        args.extend(["--working-dir", str(working_dir)])
-    if enable_reflection:
-        args.append("--enable-reflection")
-    if enable_channelz:
-        args.append("--enable-channelz")
-    if max_concurrent_streams is not None:
-        args.extend(["--max-concurrent-streams", str(max_concurrent_streams)])
-
-    if grpc_protocol_version is not None:
-        assert (
-            server_type == "grpc"
-        ), f"'grpc_protocol_version' should only be passed to gRPC server, got '{server_type}' instead."
-        args.extend(["--protocol-version", str(grpc_protocol_version)])
-
-    return ServerHandle(process=subprocess.Popen(args), host=host, port=port)
+        raise BadInput(f"Unknown server type: '{server_type}'")
