@@ -1,10 +1,10 @@
 # pylint: disable=unused-argument
 from __future__ import annotations
 
-import os
-import typing as t
-import tempfile
 import contextlib
+import os
+import tempfile
+import typing as t
 from typing import TYPE_CHECKING
 
 import psutil
@@ -12,21 +12,22 @@ import pytest
 from pytest import MonkeyPatch
 
 import bentoml
-from bentoml._internal.utils import LazyLoader
-from bentoml._internal.utils import validate_or_create_dir
-from bentoml._internal.models import ModelContext
-from bentoml._internal.configuration import CLEAN_BENTOML_VERSION
+from bentoml._internal.cloud import BentoCloudClient
+from bentoml._internal.configuration import clean_bentoml_version
 from bentoml._internal.configuration.containers import BentoMLContainer
+from bentoml._internal.models import ModelContext
+from bentoml._internal.utils.filesystem import validate_or_create_dir
+from bentoml._internal.utils.lazy_loader import LazyLoader
 
 if TYPE_CHECKING:
     import numpy as np
-    from _pytest.main import Session
-    from _pytest.nodes import Item
     from _pytest.config import Config
     from _pytest.config import ExitCode
-    from _pytest.python import Metafunc
-    from _pytest.fixtures import FixtureRequest
     from _pytest.config.argparsing import Parser
+    from _pytest.fixtures import FixtureRequest
+    from _pytest.main import Session
+    from _pytest.nodes import Item
+    from _pytest.python import Metafunc
 
     from bentoml._internal.server.metrics.prometheus import PrometheusClient
 
@@ -45,7 +46,7 @@ _RUN_GRPC_TESTS_MARKER = "--run-grpc-tests"
 
 @pytest.mark.tryfirst
 def pytest_report_header(config: Config) -> list[str]:
-    return [f"bentoml: version={CLEAN_BENTOML_VERSION}"]
+    return [f"bentoml: version={clean_bentoml_version()}"]
 
 
 @pytest.hookimpl
@@ -130,21 +131,21 @@ def _setup_deployment_mode(metafunc: Metafunc):
 
 def _setup_model_store(metafunc: Metafunc):
     """Setup dummy models for test session."""
-    with bentoml.models.create(
+    with bentoml.models._create(  # type: ignore
         "testmodel",
         module=__name__,
         signatures={},
         context=TEST_MODEL_CONTEXT,
     ):
         pass
-    with bentoml.models.create(
+    with bentoml.models._create(  # type: ignore
         "testmodel",
         module=__name__,
         signatures={},
         context=TEST_MODEL_CONTEXT,
     ):
         pass
-    with bentoml.models.create(
+    with bentoml.models._create(  # type: ignore
         "anothermodel",
         module=__name__,
         signatures={},
@@ -194,6 +195,7 @@ def _setup_test_directory() -> tuple[str, str]:
     BentoMLContainer.model_store_dir.set(models)
     BentoMLContainer.tmp_bento_store_dir.set(tmp_bentos)
     BentoMLContainer.prometheus_multiproc_dir.set(multiproc_dir)
+    BentoMLContainer.bentocloud_client.set(BentoCloudClient(""))
     return bentoml_home, multiproc_dir
 
 
@@ -220,7 +222,6 @@ def pytest_sessionstart(session: Session) -> None:
         session,
         ("PROMETHEUS_MULTIPROC_DIR", _PYTEST_MULTIPROC_DIR),
         ("BENTOML_BUNDLE_LOCAL_BUILD", "True"),
-        ("SETUPTOOLS_USE_DISTUTILS", "stdlib"),
         ("__BENTOML_DEBUG_USAGE", "False"),
         ("BENTOML_DO_NOT_TRACK", "True"),
     )
@@ -245,7 +246,6 @@ def pytest_sessionfinish(session: Session, exitstatus: int | ExitCode) -> None:
         session,
         "BENTOML_BUNDLE_LOCAL_BUILD",
         "PROMETHEUS_MULTIPROC_DIR",
-        "SETUPTOOLS_USE_DISTUTILS",
         "__BENTOML_DEBUG_USAGE",
         "BENTOML_DO_NOT_TRACK",
     )
@@ -304,6 +304,20 @@ def fixture_metrics_client() -> PrometheusClient:
 @pytest.fixture(scope="function", name="change_test_dir")
 def fixture_change_dir(request: FixtureRequest) -> t.Generator[None, None, None]:
     """A fixture to change given test directory to the directory of the current running test."""
-    os.chdir(request.fspath.dirname)  # type: ignore (bad pytest stubs)
+    os.chdir(
+        request.fspath.dirname,  # type: ignore (bad pytest stubs)
+    )
     yield
-    os.chdir(request.config.invocation_dir)  # type: ignore (bad pytest stubs)
+    os.chdir(
+        request.config.invocation_dir,  # type: ignore (bad pytest stubs)
+    )
+
+
+@pytest.fixture(scope="function", autouse=True)
+def fixture_reset_config() -> t.Generator[None, None, None]:
+    """Reset BentoML config to default."""
+    before = BentoMLContainer.config.get()
+    try:
+        yield
+    finally:
+        BentoMLContainer.config.set(before)
